import random
from nose_parameterized import parameterized
from six import iteritems

from pysbr.constants import BROWSERS, PLATFORMS


class Decorators(parameterized):
    """
    :Description: All-in-one decorators for provisioning remote selenium webdrivers.
    :Example: https://github.com/neetjn/nose-parameterized-wrapper-example
    """
    @classmethod
    def browsers(cls, platform=None, development=False):
        """
        :Description: Runs test against all browser combinatorics.
        :param platform: Platform specification for target browsers.
        :type platform: string
        :param development: If specified, will target first available browser on hub.
        :type development: bool
        """

        if development:

            # We patch for firefox, assuming Firefox is the only edge case
            # where a profile is necessary and this profile does not effect
            # any other webdriver

            return super(Decorators, cls).expand([({}, None)])

        else:

            combinations = []

            if platform:
                assert platform in [p for _, p in iteritems(pi) for pi in PLATFORMS.platforms]

            for browser in BROWSERS.browsers:

                if not browser.enabled:
                    # ignore disabled browsers
                    continue

                if platform and platform not in browser.platforms:
                    # filter out platforms
                    continue

                if platform:
                    capabilities = dict.copy(browser.capabilities)
                    capabilities.setdefault('platform', platform)
                    combinations.append([capabilities, browser.profile])
                else:
                    for _platform in browser.platforms:
                        capabilities = dict.copy(browser.capabilities)
                        capabilities.setdefault('platform', _platform)
                        combinations.append([capabilities, browser.profile])

            return super(Decorators, cls).expand([combination for combination in combinations])

    @classmethod
    def browser(cls, name, iterations=1, platform=None, capabilities=None, profile=None):  #pylint: disable=too-many-arguments
        """
        :Description: Runs test against given browser specification.
        :Warning: If a platform is not specified, selenium hub will decide first available.
        :param iterations: Number of times to run test.
        :type iterations: Integer
        :param platform: Platform to target.
        :type platform: string
        :param capabilities: Capabilities of remote webdriver.
        :type capabilities: dict
        :param profile: Profile for remote webdriver.
        :type profile: FirefoxProfile, Options, ...
        """

        browser = BROWSERS.find(name)
        capabilities = capabilities or {}

        if platform:
            assert platform in browser.platforms
            # update platform in capabilities if not specified
            capabilities.setdefault('platform', platform)

        _capabilities = dict.copy(browser.capabilities)
        _capabilities.update(capabilities)

        return super(Decorators, cls).expand(
            [(_capabilities, profile or browser.profile)]*iterations)

    @classmethod
    def random_browser(cls):
        """
        :Description: Runs test against random browser on a random available platform.
        """
        browser = random.choice([browser for browser in BROWSERS.browsers if browser.enabled])

        capabilities = dict.copy(browser.capabilities)
        capabilities.setdefault('platform', random.choice(browser.platforms))

        return super(Decorators, cls).expand([(capabilities, browser.profile)])



class PLATFORMS(object):  #pylint: disable=too-few-public-methods

    WINDOWS = {
        'default': 'WIN10',
        'legacy': 'WINDOWS'
    }

    MAC = {
        'default': 'MAC'
    }

    platforms = [WINDOWS, MAC]


class BROWSERS(object):  #pylint: disable=too-few-public-methods

    class BROWSER(object):  #pylint: disable=too-few-public-methods

        def __init__(self, name, platforms, capabilities=None, profile=None):
            """
            :Description: Base definition for browsers.
            :param name: Name of browser, will be used in capabilities.
            :type name: string
            :param platforms: Supported platforms.
            :type platforms: tuple, list
            :param capabilities: Capabilities for browser instances.
            :type capabilities: dict
            :param profile: Profile instance to be used by constructed browser.
            :type profile: FirefoxProfile, ChromeOptions, ...
            """
            self.name = name
            self.platforms = platforms
            self.capabilities = capabilities or {
                "browserName": name,
                "version": "",
                "javascriptEnabled": True
            }
            self.profile = profile
            self.enabled = False


    @staticmethod
    def find(name, enabled=True):
        """
        :Description: Find a browser by it's given name.
        :param name: Name of browser to search for.
        :type name: string
        :param enabled: Only find enabled browsers.
        :type enabled: bool
        :return: BROWSER
        """
        return next(browser for browser in BROWSERS.browsers \
            if browser.name == name and (browser.enabled if enabled else True))

    CHROME = BROWSER(
        name='chrome',
        platforms=[PLATFORMS.WINDOWS.get('default'), PLATFORMS.MAC.get('default')])

    FIREFOX = BROWSER(
        name='firefox',
        platforms=[PLATFORMS.WINDOWS.get('default'), PLATFORMS.MAC.get('default')])

    EDGE = BROWSER(
        name='edge',
        platforms=[PLATFORMS.WINDOWS.get('default')])

    SAFARI = BROWSER(
        name='safari',
        platforms=[PLATFORMS.MAC.get('default')])

    OPERA = BROWSER(
        name='opera',
        platforms=[PLATFORMS.WINDOWS.get('default'), PLATFORMS.MAC.get('default')])

    browsers = [CHROME, FIREFOX, EDGE, SAFARI, OPERA]



from copy import deepcopy
from six import iteritems

from pysbr.constants import BROWSERS, PLATFORMS


class Configure(object):

    @staticmethod
    def capabilities(name, capabilities):
        """
        :Description: Update an available browser's capabilities.
        :Warning: A browser's platform is automatically constructed when a test is decorated.
        :param capabilities: Capabilities to patch.
        :type capabilities: dict
        """
        BROWSERS.find(name, enabled=False)\
            .capabilities\
            .update(capabilities)

    @staticmethod
    def platforms(name, platforms):
        """
        :Description: Update an available browser's attributes.
        :param name: Name of browser to target.
        :type name: string
        :param platforms: List of browser's available platforms.
        :type platforms: tuple, list
        """
        available = [platform for item in PLATFORMS.platforms for _, platform in iteritems(item)]
        assert all(platform in available for platform in platforms), \
                    'An error occurred while updating; platform does not exist'
        BROWSERS.find(name, enabled=False).platforms = platforms

    @staticmethod
    def profile(name, profile):
        """
        :Description: Update an available browser's profile.
        :param name: Name of browser to target.
        :type name: string
        :param profile: Profile to override.
        :type profile: FirefoxProfile, Options, ...
        """
        BROWSERS.find(name, enabled=False).profile = deepcopy(profile)

    @staticmethod
    def enable(name):
        """
        :Description: Enable an available browser.
        :param name: Name of browser to target.
        :type name: string
        """
        BROWSERS.find(name, enabled=False).enabled = True

    @staticmethod
    def disable(name):
        """
        :Description: Disable an available browser.
        :param name: Name of browser to target.
        :type name: string
        """
        BROWSERS.find(name, enabled=False).enabled = False



from setuptools import setup, find_packages


setup(
    name='pysbr',
    description='Decorators for provisioning tests with remote webdrivers.',
    version='0.0.1',
    url='https://neetjn.github.io/py-selenium-browser',
    author='John Nolette',
    author_email='john@neetgroup.net',
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.6'
    ],
    install_requires=[
        'nose',
        'nose_parameterized',
        'six'
    ],
    packages=['pysbr']
)
